@@ -17,10 +17,6 @@
  * under the License.
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
 #include <arrow-glib/array.hpp>
 #include <arrow-glib/chunked-array.hpp>
 #include <arrow-glib/data-type.hpp>
@@ -29,6 +25,8 @@
 #include <arrow-glib/field.hpp>
 #include <arrow-glib/type.hpp>
 
+#include <arrow/c/bridge.h>
+
 G_BEGIN_DECLS
 
 /**
@@ -80,6 +78,9 @@ G_BEGIN_DECLS
  * #GArrowLargeStringDataType is a class for the 64-bit offsets UTF-8
  * encoded string data type.
  *
+ * #GArrowTemporalDataType is an abstract class for temporal related data type
+ * such as #GArrowDate32DataType.
+ *
  * #GArrowDate32DataType is a class for the number of days since UNIX
  * epoch in the 32-bit signed integer data type.
  *
@@ -96,6 +97,18 @@ G_BEGIN_DECLS
  * #GArrowTime64DataType is a class for the number of microseconds or
  * nanoseconds since midnight in the 64-bit signed integer data type.
  *
+ * #GArrowIntervalDataType is an abstract class for interval related
+ * data type such as #GArrowMonthIntervalDataType.
+ *
+ * #GArrowMonthIntervalDataType is a class for the month intarval data
+ * type.
+ *
+ * #GArrowDayTimeIntervalDataType is a class for the day time intarval
+ * data type.
+ *
+ * #GArrowMonthDayNanoIntervalDataType is a class for the month day
+ * nano intarval data type.
+ *
  * #GArrowDecimalDataType is a base class for the decimal data types.
  *
  * #GArrowDecimal128DataType is a class for the 128-bit decimal data type.
@@ -200,6 +213,58 @@ garrow_data_type_class_init(GArrowDataTypeClass *klass)
   g_object_class_install_property(gobject_class, PROP_DATA_TYPE, spec);
 }
 
+/**
+ * garrow_data_type_import:
+ * @c_abi_schema: (not nullable): A `struct ArrowSchema *`.
+ * @error: (nullable): Return location for a #GError or %NULL.
+ *
+ * Returns: (transfer full) (nullable): An imported #GArrowDataType on success,
+ *   %NULL on error.
+ *
+ *   You don't need to release the passed `struct ArrowSchema *`,
+ *   even if this function reports an error.
+ *
+ * Since: 6.0.0
+ */
+GArrowDataType *
+garrow_data_type_import(gpointer c_abi_schema, GError **error)
+{
+  auto arrow_data_type_result =
+    arrow::ImportType(static_cast<ArrowSchema *>(c_abi_schema));
+  if (garrow::check(error, arrow_data_type_result, "[data-type][import]")) {
+    return garrow_data_type_new_raw(&(*arrow_data_type_result));
+  } else {
+    return NULL;
+  }
+}
+
+/**
+ * garrow_data_type_export:
+ * @data_type: A #GArrowDataType.
+ * @error: (nullable): Return location for a #GError or %NULL.
+ *
+ * Returns: (transfer full) (nullable): An exported #GArrowDataType as
+ *   `struct ArrowStruct *` on success, %NULL on error.
+ *
+ *   It should be freed with the `ArrowSchema::release` callback then
+ *   g_free() when no longer needed.
+ *
+ * Since: 6.0.0
+ */
+gpointer
+garrow_data_type_export(GArrowDataType *data_type, GError **error)
+{
+  const auto arrow_data_type = garrow_data_type_get_raw(data_type);
+  auto c_abi_schema = g_new(ArrowSchema, 1);
+  auto status = arrow::ExportType(*arrow_data_type, c_abi_schema);
+  if (garrow::check(error, status, "[data-type][export]")) {
+    return c_abi_schema;
+  } else {
+    g_free(c_abi_schema);
+    return NULL;
+  }
+}
+
 /**
  * garrow_data_type_equal:
  * @data_type: A #GArrowDataType.
@@ -925,9 +990,24 @@ garrow_large_string_data_type_new(void)
 }
 
 
+G_DEFINE_ABSTRACT_TYPE(GArrowTemporalDataType,
+                       garrow_temporal_data_type,
+                       GARROW_TYPE_FIXED_WIDTH_DATA_TYPE)
+
+static void
+garrow_temporal_data_type_init(GArrowTemporalDataType *object)
+{
+}
+
+static void
+garrow_temporal_data_type_class_init(GArrowTemporalDataTypeClass *klass)
+{
+}
+
+
 G_DEFINE_TYPE(GArrowDate32DataType,
               garrow_date32_data_type,
-              GARROW_TYPE_DATA_TYPE)
+              GARROW_TYPE_TEMPORAL_DATA_TYPE)
 
 static void
 garrow_date32_data_type_init(GArrowDate32DataType *object)
@@ -962,7 +1042,7 @@ garrow_date32_data_type_new(void)
 
 G_DEFINE_TYPE(GArrowDate64DataType,
               garrow_date64_data_type,
-              GARROW_TYPE_DATA_TYPE)
+              GARROW_TYPE_TEMPORAL_DATA_TYPE)
 
 static void
 garrow_date64_data_type_init(GArrowDate64DataType *object)
@@ -997,7 +1077,7 @@ garrow_date64_data_type_new(void)
 
 G_DEFINE_TYPE(GArrowTimestampDataType,
               garrow_timestamp_data_type,
-              GARROW_TYPE_DATA_TYPE)
+              GARROW_TYPE_TEMPORAL_DATA_TYPE)
 
 static void
 garrow_timestamp_data_type_init(GArrowTimestampDataType *object)
@@ -1052,7 +1132,7 @@ garrow_timestamp_data_type_get_unit(GArrowTimestampDataType *timestamp_data_type
 
 G_DEFINE_ABSTRACT_TYPE(GArrowTimeDataType,
                        garrow_time_data_type,
-                       GARROW_TYPE_DATA_TYPE)
+                       GARROW_TYPE_TEMPORAL_DATA_TYPE)
 
 static void
 garrow_time_data_type_init(GArrowTimeDataType *object)
@@ -1219,6 +1299,140 @@ garrow_time64_data_type_new(GArrowTimeUnit unit, GError **error)
 }
 
 
+G_DEFINE_ABSTRACT_TYPE(GArrowIntervalDataType,
+                       garrow_interval_data_type,
+                       GARROW_TYPE_TEMPORAL_DATA_TYPE)
+
+static void
+garrow_interval_data_type_init(GArrowIntervalDataType *object)
+{
+}
+
+static void
+garrow_interval_data_type_class_init(GArrowIntervalDataTypeClass *klass)
+{
+}
+
+/**
+ * garrow_interval_data_type_get_interval_type:
+ * @type: The #GArrowIntervalDataType.
+ *
+ * Returns: The interval type of the given @type.
+ *
+ * Since: 7.0.0
+ */
+GArrowIntervalType
+garrow_interval_data_type_get_interval_type(GArrowIntervalDataType *type)
+{
+  const auto arrow_data_type = garrow_data_type_get_raw(GARROW_DATA_TYPE(type));
+  const auto arrow_interval_type =
+    std::static_pointer_cast<arrow::IntervalType>(arrow_data_type);
+  return garrow_interval_type_from_raw(arrow_interval_type->interval_type());
+}
+
+
+G_DEFINE_TYPE(GArrowMonthIntervalDataType,
+              garrow_month_interval_data_type,
+              GARROW_TYPE_INTERVAL_DATA_TYPE)
+static void
+garrow_month_interval_data_type_init(GArrowMonthIntervalDataType *object)
+{
+}
+
+static void
+garrow_month_interval_data_type_class_init(
+  GArrowMonthIntervalDataTypeClass *klass)
+{
+}
+
+/**
+ * garrow_month_interval_data_type_new:
+ *
+ * Returns: The newly created month interval data type.
+ *
+ * Since: 7.0.0
+ */
+GArrowMonthIntervalDataType *
+garrow_month_interval_data_type_new(void)
+{
+  auto arrow_data_type = arrow::month_interval();
+
+  auto data_type = g_object_new(GARROW_TYPE_MONTH_INTERVAL_DATA_TYPE,
+                                "data-type", &arrow_data_type,
+                                NULL);
+  return GARROW_MONTH_INTERVAL_DATA_TYPE(data_type);
+}
+
+
+G_DEFINE_TYPE(GArrowDayTimeIntervalDataType,
+              garrow_day_time_interval_data_type,
+              GARROW_TYPE_INTERVAL_DATA_TYPE)
+
+static void
+garrow_day_time_interval_data_type_init(GArrowDayTimeIntervalDataType *object)
+{
+}
+
+static void
+garrow_day_time_interval_data_type_class_init(
+  GArrowDayTimeIntervalDataTypeClass *klass)
+{
+}
+
+/**
+ * garrow_day_time_interval_data_type_new:
+ *
+ * Returns: The newly created day time interval data type.
+ *
+ * Since: 7.0.0
+ */
+GArrowDayTimeIntervalDataType *
+garrow_day_time_interval_data_type_new(void)
+{
+  auto arrow_data_type = arrow::day_time_interval();
+
+  auto data_type = g_object_new(GARROW_TYPE_DAY_TIME_INTERVAL_DATA_TYPE,
+                                "data-type", &arrow_data_type,
+                                NULL);
+  return GARROW_DAY_TIME_INTERVAL_DATA_TYPE(data_type);
+}
+
+
+G_DEFINE_TYPE(GArrowMonthDayNanoIntervalDataType,
+              garrow_month_day_nano_interval_data_type,
+              GARROW_TYPE_INTERVAL_DATA_TYPE)
+
+static void
+garrow_month_day_nano_interval_data_type_init(
+  GArrowMonthDayNanoIntervalDataType *object)
+{
+}
+
+static void
+garrow_month_day_nano_interval_data_type_class_init(
+  GArrowMonthDayNanoIntervalDataTypeClass *klass)
+{
+}
+
+/**
+ * garrow_month_day_nano_interval_data_type_new:
+ *
+ * Returns: The newly created month day nano interval data type.
+ *
+ * Since: 7.0.0
+ */
+GArrowMonthDayNanoIntervalDataType *
+garrow_month_day_nano_interval_data_type_new(void)
+{
+  auto arrow_data_type = arrow::month_day_nano_interval();
+
+  auto data_type = g_object_new(GARROW_TYPE_MONTH_DAY_NANO_INTERVAL_DATA_TYPE,
+                                "data-type", &arrow_data_type,
+                                NULL);
+  return GARROW_MONTH_DAY_NANO_INTERVAL_DATA_TYPE(data_type);
+}
+
+
 G_DEFINE_ABSTRACT_TYPE(GArrowDecimalDataType,
                        garrow_decimal_data_type,
                        GARROW_TYPE_FIXED_SIZE_BINARY_DATA_TYPE)
@@ -1237,20 +1451,29 @@ garrow_decimal_data_type_class_init(GArrowDecimalDataTypeClass *klass)
  * garrow_decimal_data_type_new:
  * @precision: The precision of decimal data.
  * @scale: The scale of decimal data.
+ * @error: (nullable): Return location for a #GError or %NULL.
  *
- * Returns: The newly created decimal data type.
+ * Returns: (nullable):
+ *   The newly created decimal data type on success, %NULL on error.
+ *
+ *   #GArrowDecimal256DataType is used if @precision is larger than
+ *   garrow_decimal128_data_type_max_precision(),
+ *   #GArrowDecimal128DataType is used otherwise.
  *
  * Since: 0.10.0
- *
- * Deprecated: 0.12.0:
- *   Use garrow_decimal128_data_type_new() instead.
  */
 GArrowDecimalDataType *
 garrow_decimal_data_type_new(gint32 precision,
-                             gint32 scale)
+                             gint32 scale,
+                             GError **error)
 {
-  auto decimal128_data_type = garrow_decimal128_data_type_new(precision, scale);
-  return GARROW_DECIMAL_DATA_TYPE(decimal128_data_type);
+  if (precision <= garrow_decimal128_data_type_max_precision()) {
+    return GARROW_DECIMAL_DATA_TYPE(
+      garrow_decimal128_data_type_new(precision, scale, error));
+  } else {
+    return GARROW_DECIMAL_DATA_TYPE(
+      garrow_decimal256_data_type_new(precision, scale, error));
+  }
 }
 
 /**
@@ -1321,22 +1544,30 @@ garrow_decimal128_data_type_max_precision()
  * garrow_decimal128_data_type_new:
  * @precision: The precision of decimal data.
  * @scale: The scale of decimal data.
+ * @error: (nullable): Return location for a #GError or %NULL.
  *
- * Returns: The newly created 128-bit decimal data type.
+ * Returns: (nullable):
+ *   The newly created 128-bit decimal data type on success, %NULL on error.
  *
  * Since: 0.12.0
  */
 GArrowDecimal128DataType *
 garrow_decimal128_data_type_new(gint32 precision,
-                                gint32 scale)
+                                gint32 scale,
+                                GError **error)
 {
-  auto arrow_data_type = arrow::decimal128(precision, scale);
-
-  auto data_type =
-    GARROW_DECIMAL128_DATA_TYPE(g_object_new(GARROW_TYPE_DECIMAL128_DATA_TYPE,
-                                             "data-type", &arrow_data_type,
-                                             NULL));
-  return data_type;
+  auto arrow_data_type_result = arrow::Decimal128Type::Make(precision, scale);
+  if (garrow::check(error,
+                    arrow_data_type_result,
+                    "[decimal128-data-type][new]")) {
+    auto arrow_data_type = *arrow_data_type_result;
+    return GARROW_DECIMAL128_DATA_TYPE(
+      g_object_new(GARROW_TYPE_DECIMAL128_DATA_TYPE,
+                   "data-type", &arrow_data_type,
+                   NULL));
+  } else {
+    return NULL;
+  }
 }
 
 
@@ -1371,22 +1602,30 @@ garrow_decimal256_data_type_max_precision()
  * garrow_decimal256_data_type_new:
  * @precision: The precision of decimal data.
  * @scale: The scale of decimal data.
+ * @error: (nullable): Return location for a #GError or %NULL.
  *
- * Returns: The newly created 256-bit decimal data type.
+ * Returns: (nullable):
+ *   The newly created 256-bit decimal data type on success, %NULL on error.
  *
  * Since: 3.0.0
  */
 GArrowDecimal256DataType *
 garrow_decimal256_data_type_new(gint32 precision,
-                                gint32 scale)
+                                gint32 scale,
+                                GError **error)
 {
-  auto arrow_data_type = arrow::decimal256(precision, scale);
-
-  auto data_type =
-    GARROW_DECIMAL256_DATA_TYPE(g_object_new(GARROW_TYPE_DECIMAL256_DATA_TYPE,
-                                             "data-type", &arrow_data_type,
-                                             NULL));
-  return data_type;
+  auto arrow_data_type_result = arrow::Decimal256Type::Make(precision, scale);
+  if (garrow::check(error,
+                    arrow_data_type_result,
+                    "[decimal256-data-type][new]")) {
+    auto arrow_data_type = *arrow_data_type_result;
+    return GARROW_DECIMAL256_DATA_TYPE(
+      g_object_new(GARROW_TYPE_DECIMAL256_DATA_TYPE,
+                   "data-type", &arrow_data_type,
+                   NULL));
+  } else {
+    return NULL;
+  }
 }
 
 
@@ -1929,6 +2168,9 @@ garrow_data_type_new_raw(std::shared_ptr<arrow::DataType> *arrow_data_type)
   case arrow::Type::type::DICTIONARY:
     type = GARROW_TYPE_DICTIONARY_DATA_TYPE;
     break;
+  case arrow::Type::type::MAP:
+    type = GARROW_TYPE_MAP_DATA_TYPE;
+    break;
   case arrow::Type::type::DECIMAL128:
     type = GARROW_TYPE_DECIMAL128_DATA_TYPE;
     break;
